@@ -109,7 +109,7 @@ impl Executor for ThreadPoolExecutor {
 
 #[cfg(test)]
 mod tests {
-    extern crate env_logger;
+    use env_logger;
 
     use super::*;
     use std::time::Duration;
